﻿using System;
using System.IO.Ports;
using System.Text;
using System.Windows.Forms;

namespace SerialTester
{
    public partial class Form1 : Form
    {
        SerialPort ComPort = null;

        public Form1()
        {
            InitializeComponent();
            showPacket();
            string[] theSerialPorts = SerialPort.GetPortNames();
            cmbPorts.Items.AddRange(theSerialPorts);
            cmbPorts.Text = cmbPorts.Items[0].ToString();
        }

        private void cmdSendPacket_Click(object sender, EventArgs e)
        {
            byte[] packet = createPacket();
            ComPort.Write(packet, 0, packet.Length);
        }

        private byte[] createPacket()
        {
            if (rbOn.Checked) return new byte[] {97, 0};
            if (rbOff.Checked) return new byte[] {98, 0};
            byte[] strBytes = Encoding.ASCII.GetBytes(txtLCD.Text);
            byte[] packet = new byte[2 + txtLCD.Text.Length];
            packet[0] = 116;
            packet[1] = (byte)txtLCD.Text.Length;
            Buffer.BlockCopy(strBytes, 0, packet, 2, strBytes.Length);
            return packet;
        }

        void showPacket()
        {
            lblPreview.Text = "";
            byte[] packet = createPacket();
            foreach (byte b in packet)
            {
                lblPreview.Text += b.ToString() + " ";
            }
        }

        private void txtLCD_TextChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void rbOn_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void rbOff_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void rbText_CheckedChanged(object sender, EventArgs e)
        {
            showPacket();
        }

        private void cmdSendRandom_Click(object sender, EventArgs e)
        {
            try
            {
                string[] strBytes = txtRandomBytes.Text.Split(' ');
                foreach (string str in strBytes)
                {
                    if (str.Length > 0)
                    {
                        byte[] b = new byte[1];
                        b[0] = Byte.Parse(str);
                        ComPort.Write(b, 0, 1);
                    }
                }
            } catch
            {
                MessageBox.Show("Nieprawidłowy ciąg bajtów", "Błąd", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void cmdOpen_Click(object sender, EventArgs e)
        {
            try
            {
                ComPort = new SerialPort(cmbPorts.SelectedItem.ToString());
                ComPort.BaudRate = 9600;
                ComPort.Parity = Parity.None;
                ComPort.StopBits = StopBits.One;
                ComPort.DataBits = 8;
                ComPort.Handshake = Handshake.None;
                ComPort.Open();
                ComPort.DataReceived += OnSerialDataReceived;
                cmdSendPacket.Enabled = true;
                cmdSendRandom.Enabled = true;
                cmdOpen.Enabled = false;
                cmdClose.Enabled = true;
            } catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Błąd otwierania portu", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void OnSerialDataReceived(object sender, SerialDataReceivedEventArgs args)
        {
            byte[] data = new byte[ComPort.BytesToRead];
            ComPort.Read(data, 0, data.Length);
            foreach(byte b in data) {
                txtReceived.Invoke((MethodInvoker)(() => txtReceived.Text += b.ToString() + " "));
            }
        }

        private void cmdClose_Click(object sender, EventArgs e)
        {
            cmdSendPacket.Enabled = false;
            cmdSendRandom.Enabled = false;
            cmdOpen.Enabled = true;
            cmdClose.Enabled = false;
            if (ComPort != null) ComPort.Close();
        }

        private void cmdClear_Click(object sender, EventArgs e)
        {
            txtReceived.Text = "";
        }
    }
}
